# %%
from plotly_resampler import FigureWidgetResampler
from plotly.subplots import make_subplots
import plotly.graph_objects as go
import numpy as np

from wrpll_simulation.config import Timesim_Config, PI_Config
from wrpll_simulation.timesim import WRPLL_Timesim

# %%
KP, KI = 6, 2
sim_config = Timesim_Config(
    timestep_size=1e-11,
    sim_length=500_000_000,
    helper_PI=PI_Config(KP=KP, KI=KI),
    main_PI=PI_Config(KP=KP, KI=KI),
    has_jitter=False,
    step_input_time=10e-6,
    step_frequency=6.25,
    step_phase=0,
)
sim = WRPLL_Timesim(sim_config, np.random.default_rng(1))

fig = FigureWidgetResampler(make_subplots(rows=2))
# ROW 1
fig.add_trace(
    go.Scattergl(name="main - gtx freq diff"),
    hf_x=sim.time,
    hf_y=sim.freq_diff,
    row=1,
    col=1,
)

# ROW 2
fig.add_trace(
    go.Scattergl(name="main - gtx phase diff"),
    hf_x=sim.time,
    hf_y=sim.phase_diff,
    row=2,
    col=1,
)


title = (
    f'step freq = {sim_config.step_frequency}Hz, '
    f'step phase = {sim_config.step_phase}° | '
    f'KP = {KP}, KI = {KI}'
)

fig.update_layout(
    xaxis2=dict(title="Time (sec)", exponentformat="SI"),
    yaxis1=dict(title="Frequency (Hz)"),
    yaxis2=dict(title="Phase (degree)"),
    height=950,
    showlegend=True,
    title_text=title,
    legend=dict(
        orientation="h",
        yanchor="bottom",
        y=1.02,
        xanchor="right",
        x=1,
    ),
)

fig

# %%
