from numba.core.types import *
from numba.experimental import jitclass


@jitclass
class PI_Config(object):
    KP: int64
    KI: int64

    def __init__(self, KP, KI):
        self.KP = KP
        self.KI = KI


@jitclass
class Timesim_Config(object):
    timestep_size: float32
    sim_length: int64
    helper_PI: PI_Config
    main_PI: PI_Config
    has_jitter: bool

    step_input_time: float64
    step_frequency: float64
    step_phase: float64

    # preset
    beating_period: int64
    blind_period: int16
    adpll_limit: int32

    gtx_init_phase: float64
    gtx_init_freq: float64
    gtx_jitter: float64

    helper_init_phase: float64
    main_init_phase: float64
    helper_init_freq: float64
    main_init_freq: float64
    dcxo_jitter: float64

    irq_delay: float64
    i2c_comm_delay: float64
    dcxo_settling_delay: float64

    # jitclass does not support **kwargs
    def __init__(
        self,
        timestep_size: float32,
        sim_length: int64,
        helper_PI: PI_Config,
        main_PI: PI_Config,
        has_jitter: bool,
        step_input_time: float64,
        step_frequency: float64,
        step_phase: float64,
    ):
        # jitter << timestep_size
        # otherwise simulation will add negative phase to Phase_Accumlator
        self.timestep_size = timestep_size
        self.sim_length = sim_length
        self.helper_PI = helper_PI
        self.main_PI = main_PI
        self.has_jitter = has_jitter

        # step input
        self.step_input_time = step_input_time
        self.step_frequency = step_frequency
        self.step_phase = step_phase

        # preset
        self.beating_period = 32768
        self.blind_period = 200
        self.adpll_limit = 8161512

        self.gtx_init_phase = 0.0
        self.gtx_init_freq = 125_000_000
        self.gtx_jitter = 200e-17

        self.helper_init_phase = 0.0
        self.main_init_phase = 0.0
        self.helper_init_freq = 125_000_000 * (1 - (1 / self.beating_period))
        self.main_init_freq = 125_000_000
        self.dcxo_jitter = 95e-17

        # hardware delay
        self.irq_delay = 2e-6  # ~2us for the interrupt handling
        self.i2c_comm_delay = 85.6e-6
        self.dcxo_settling_delay = 100e-6
